// Copyright (C) 2008
// by all contributors.

// This file is part of SCNSL.

// SCNSL is free software:
// you can redistribute it and/or modify it under the terms of the
// GNU Lesser General Public License as published by the Free Software
// Foundation, either version 3 of the License, or (at your option)
// any later version.

// SCNSL is distributed in the hope
// that it will be useful, but WITHOUT ANY WARRANTY;
// without even the implied warranty of MERCHANTABILITY or FITNESS
// FOR A PARTICULAR PURPOSE.
// See the GNU Lesser General Public License for more details.

// You should have received a copy of the GNU Lesser General Public
// License along with SCNSL,
// in a file named LICENSE.txt.
// If not, see <http://www.gnu.org/licenses/>.
#include "../core/Packet_t.hh"
#include "../core/Node_t.hh"

#include "Environment_if_t.hh"

#ifndef SCNSL__UTILS__DEFAULTENVIRONMENT_T_HH
#define SCNSL__UTILS__DEFAULTENVIRONMENT_T_HH


namespace Scnsl { namespace Utils {


class DefaultEnvironment_t:
    public Environment_if_t
{
public:

    typedef Scnsl::Core::node_properties_t node_properties_t;
    typedef Scnsl::Core::propagation_t propagation_t;
    typedef Scnsl::Core::position_t position_t;

    DefaultEnvironment_t(const double alpha = 1,const propagation_t propagation = 1);

    ~DefaultEnvironment_t();

    static void createInstance(const double alpha = 1,const propagation_t propagation = 1);

    /// @brief Gets Bit error rate.
    ///
    /// @param sp source node properties.
    /// @param rp destination node properties.
    ///
    virtual double getBitErrorRate(const node_properties_t & sp,
                                   const node_properties_t & rp);

    /// @brief Gets receiver power.
    ///
    /// @param sp source node properties.
    /// @param rp destination node properties.
    ///
    virtual double getReceiverPower( const node_properties_t & sp,
                                     const node_properties_t & rp);

    /// @brief Gets maximum delay.
    ///
    /// @param sp source node properties.
    /// @param rp destination node properties.
    ///
    virtual double getMaxDelay( const node_properties_t & sp,
                                const node_properties_t & rp);

    /// @brief Gets propagation value.
    ///
    ///
    virtual Scnsl::Core::propagation_t getPropagation();

protected:
    /// @brief The attenuation exponent.
    const double _ALPHA;
    const propagation_t _PROPAGATION;
};

} }

#endif
